#!/usr/bin/perl -w
package SmartClient::Config;
use strict;
use warnings;

use Log::Log4perl qw(:easy);
use SmartClient::Util;

=head1 NAME

=begin comment

Name des Moduls,Grobkonzept

=end comment

B<SmartClient::Config>

=head1 SYNOPSIS

=begin comment

  kurze Beschreibung, was das Programm macht, Feinkonzept

=end comment

Objekt zum einlesen und Abfragen von Configurationsparametern.

 SmartClient::Config->new("Konfiguration-Datei"[, "Konfig Datei 2", ....]);

=head1 DESCRIPTION

=begin comment

  muss fuer das Grobkonzept ausgefuellt werden

=end comment

=head2 Allg. Beschreibung und Aufgaben

Modul liest eine Konfigurationsdatei aus, um eine zentrale Verwaltung von Funktionsparametern für benutzte Perl-Programme zu ermöglichen.

=head2 Funktionsweise / Design / Architektur / Sprache

=over 4

=item Funktionsweise

B<SmartClient::Config> bildet eine Instanz einer Konfigurationsdatei, wenn diese gültig, d.h. vorhanden und lesbar ist. Die Konfigurationsdatei hat dabei folgendes Format:

OPTION = WERT.

=item Architektur

Linux i386


=item Sprache

  Perl

=back


=head2 Sicherheitskonzept

keine Angabe

=head2 Einordnung / Abhängigkeiten / Voraussetzungen

keine

=head2 Interfaces(Programm/Benutzer)

Konfigurations-Dateinname

=cut

=head2 CVS Version

$Id: Config.pm 7833 2012-03-16 07:08:47Z mschausten $

=cut

my $VERSION = '1.1';

=head2 Installation und sonstige Hinweise

=begin comment

Feinkonzept

=end comment

Modul muß im @INC -Pfad stehen.

=head1 AUTHOR

=begin comment

name and e-mail

=end comment

Frank Balzer mailto:frank.balzer@suse.de

SuSE Linux Solutions AG mailto: rheinruhr@suse.de

=head1 SEE ALSO


=cut

=head1 exportierte Module, Methoden, Variablen

=over 4

=item Variable

=back

=cut

# define constants

my $DEFAULT_CONFIGFILE = "/etc/smartclient/base";

=head1 Modul

=begin comment

Beschreibung des Moduls

=end comment

S<  >B<SmartClient::Config> { I<FILE> =E<gt> I<config-file>

  # Beispiel

  use SmartClient::Config;
  
  $obj = SmartClient::Config->new();  # Öffnet /etc/smartclient/base

  $obj1 = SmartClient::Config->new("/etc/SmartClient/config.dat"); 

  foreach $option (keys(%$obj1) 
  {
      print $obj1->{$option} . "\n";
  }

=cut

# constructor

sub new
{
    my ($package, @files) = @_;
    if ( $#files == -1 ) {
        push @files, $DEFAULT_CONFIGFILE;
    }
    if ( !Log::Log4perl->initialized() ) {
        Log::Log4perl->easy_init($ERROR);

        #TODO add syslog
    }
    my $logger = Log::Log4perl->get_logger();

    my $data = {};

    foreach my $file (@files) {

        # try to open config file
        if ( !open( my $fh, '<', $file ) ) {
            $logger->warn( "can\'t open config file" . $file );
        } else {
            while (<$fh>) {
                chomp;
                if (/^#/) {
                    next;
                }
                if ( $_ =~ /=/g ) {
                    ## splitting in 2 fields at maximum
                    my ( $option, $value ) = split( /=/, $_, 2 );
                    $option =~ s/^\s+//g;
                    $option =~ s/\s+$//g;
                    $option =~ s/\"+//g;
                    $value  =~ s/^\s+//g;
                    $value  =~ s/\s+$//g;
                    $value  =~ s/\"+//g;

                    if ( length($option) ) {
                        $data->{$option} = $value;
                    }
                }
            }
            close $fh;
        }
    }

    return bless( $data, $package );
}

=head2 getUndefined

    checks a attribute list for existence

=over 4

=item Parameter

    - list of attributes

=item Return

    list of undefined attributes

=back

=cut

sub getUndefined
{
    my ($self, @attributes) = @_;

    return getUndefinedHashKeys( $self, \@attributes );
}

=head2 require

    checks a attribute list for existence.
    if attributes does not exist,
    this will be logged and program dies.

=over 4

=item Parameter

    - list of attributes

=item Return

    true or program dies

=back

=cut

sub require
{
    my ( $self, @attributes ) = @_;

    my $logger = Log::Log4perl->get_logger();
    my @config_undefined_attributes = $self->getUndefined(@attributes);

    if (@config_undefined_attributes) {
        $logger->logdie( "missing config variables "
              . join( ", ", @config_undefined_attributes )
              . ". Exiting" );
    }

    # returns 1 if all required attributes are present,
    # otherwise it has already died
    return 1;
}

=head2 getKeys

    returns the keys of the config file

=over 4

=item Parameter


=item Return

    value of the configuration attribute or undef on error

=back

=cut

sub getKeys
{
    my ( $self ) = @_;

    return keys %{$self};
}

=head2 get

    returns the value of a configuration attribute.
    Logs an error, if attribute is not defined

=over 4

=item Parameter

    - attribute name
    - [optional (default: false)]

=item Return

    value of the configuration attribute or undef on error

=back

=cut

sub get
{
    my ( $self, $attribute, $optional ) = @_;

    my $value = $self->{$attribute};
    if ( ( not $optional ) and ( not defined($value) ) ) {
        my $logger = Log::Log4perl->get_logger();
        $logger->error("failed to get config attribute $attribute");
    }

    return $value;
}

=head2 getWithDefault

    returns the value of a configuration attribute, or the given
    default value, if the wanted attribute is not defined.

    Logs a warning, if wanted attribute is not defined and prefix is defined.

=over 4

=item Parameter

    - attribute name
    - default value to return when wanted attribute is not defined
    - prefix to use in warnings log message

=item Return

    value of the configuration attribute or default value

=back

=cut

sub getWithDefault
{
    my ( $self, $attribute, $default, $prefix ) = @_;

    my $value = $self->{$attribute};
    if ( !defined $value ) {
        my $logger = Log::Log4perl->get_logger();
        if ($prefix) {
            $logger->warn( "[$prefix] config parameter $attribute not set, "
                  . "assuming $default" );
        }
        $value = $default;
    }
    return $value;
}

=head2 isEnabled

    returns true, if a value is defined and set to "1", "yes" or "true"

=over 4

=item Parameter

    - attribute name

=item Return

    returns true, if a value is enabled ("1", "yes" or "true")

=back

=cut

sub isEnabled
{
    my ($self, $attribute) = @_;

    my $value = $self->{$attribute};

    if( $value ) {
        $value = lc($value);
   
        if( ( $value eq "1" ) || ( $value eq "yes" ) || ( $value eq "true" ) ) {
            return 1;
        }
    }

    return 0;
}

=head2 isDisabled

    returns true, if a value is disabled (undefined or set to 0)

=over 4

=item Parameter

    - attribute name

=item Return

    returns true, if a value is disabled (undefined or set to 0)

=back

=cut

sub isDisabled
{
    my ( $self, $attribute ) = @_;

    my $value = $self->{$attribute};

    # return true, if value is not defined or 0
    # (config stores value as string,
    # therefore comparison between "0" and 0 is required)
    return ( ( !$value ) || ( $value eq "0" ) );
}

1;

