#!/usr/bin/perl -w
use strict;
use warnings;

package SmartClient::IP;

our $VERSION = '1.0';

=encoding utf8

=head1 NAME

C<SmartClient::IP>

=head2 SVN Version

 $Id: IP.pm 7479 2011-10-07 12:45:47Z FSchumacher $

=head1 SYNOPSIS

Modul for IP-Address handling.
Supports single IP addresses, IP network and IP ranges.

  $ip = SmartClient::IP->new( "192.168.1.10" ) || die;
  $ip = SmartClient::IP->new( "192.168.1.0", "255.255.255.0" ) || die;
  $ip = SmartClient::IP->new( "192.168.1.10", "255.255.255.0" ) || die;
  $ip = SmartClient::IP->new( "192.168.1.10 - 192.168.1.20" ) || die;
  $ip = SmartClient::IP->new( "192.168.1.10 - 192.168.1.20", "255.255.255.0" ) || die;



=head1 DESCRIPTION

=head2 Purpose

Modul provides IP related functions.

=head2 Realisation

B<SmartClient::IP> creates an instance for IP-Addresses.
Currently only IPv4 is supported.

=head2 Error handling

on error, constructor and other functions return undef.
Last error code and message are provides by the functions errno() and error() 
(currently not implemented)


=head2 Context and Dependecies

Provides a generalized interface to Net::IP and Net::Netmask


=head2 Interfaces


=head2 Status


=head2 Installation and other notes

none

=head1 AUTHOR

  Frank Balzer
  Jörg Steffens

=head1 SEE ALSO

  Net::IP
  Net::Netmask

=cut

#
# used modules
#
use Data::Dumper;
use Log::Log4perl;
use Net::IP qw( ip_inttobin ip_bintoip );
use Net::Netmask;

my $allowed_ip_range_regex_string = '([0-9.]+(?:\s*[-/]\s*[0-9.]+)?)';
my $allowed_ip_range_regex        = qr{$allowed_ip_range_regex_string};

#use Math::BigInt;
#use SmartClient::CheckUtil qw( chk_ip );

=head1 methods

=head2 new()

constructor

 IP address | IP address range, [netmask]           => network,     netmask,       first IP,     last IP
 --------------------------------------------------------------------------------------------------------------
 ( "192.168.1.10" )                                 => undef,       undef,         192.168.1.10, 192.168.1.10
 ( "192.168.1.0", "255.255.255.0" )                 => 192.168.1.0, 255.255.255.0, 192.168.1.1,  192.168.1.254
 ( "192.168.1.10", "255.255.255.0" )                => 192.168.1.0, 255.255.255.0, 192.168.1.10, 192.168.1.10
 ( "192.168.1.10 - 192.168.1.20" )                  => undef,       undef,         192.168.1.10, 192.168.1.20
 ( "192.168.1.10 - 192.168.1.20", "255.255.255.0")  => 192.168.1.0, 255.255.255.0, 192.168.1.10, 192.168.1.20

=over 4

=item Parameter

    - string: IP address ("IP") or IP range ("IP - IP")
    - string: netmask

=item Return

    object reference or undef on error

=back

=cut

sub new
{
    my $package = shift;

    my $ip_range = shift;
    my $netmask  = shift;

    my $data = {
        'ip_range'   => undef,
        'ip_network' => undef,
        'first_ip'   => undef,
        'last_ip'    => undef,
        'network'    => undef,
        'netmask'    => undef,
        'ip_version' => 4,
        'errno'      => 0,
        'error'      => undef,
    };

    # create Net::IP instance
    if ( !$ip_range ) {
        my $logger = Log::Log4perl->get_logger();
        $logger->warn(
            "IP Range was not given as a parameter. Can not create an IP Object."
        );
        return;
    }
    if ( $ip_range !~ m/^$allowed_ip_range_regex$/ ) {
        my $logger = Log::Log4perl->get_logger();
        $logger->warn(
            "IP Range \"$ip_range\" is not in the right format, it has to be: '$allowed_ip_range_regex_string'"
        );
    }

    $data->{'ip_range'} = Net::IP->new($ip_range) or return;

    $data->{'first_ip'} = $data->{'ip_range'}->ip();
    $data->{'last_ip'}  = $data->{'ip_range'}->last_ip();

    if ($netmask) {
        $data->{'netmask'} = $netmask;
        $data->{'ip_network'} =
          new2 Net::Netmask( $data->{'first_ip'}, $data->{'netmask'} )
          or return;
        $data->{'network'} = $data->{'ip_network'}->base();

        # if first parameter has been a network IP address
        # IP range will be set to all valid IP address of this network
        if (    ( $data->{'first_ip'} eq $data->{'last_ip'} )
            and ( $data->{'first_ip'} eq $data->{'network'} )
            and $data->{'netmask'} ne '255.255.255.255' )
        {
            $data->{'first_ip'} = $data->{'ip_network'}->nth(1);
            $data->{'last_ip'}  = $data->{'ip_network'}->nth(-2);
            $data->{'ip_range'} =
              Net::IP->new( $data->{'first_ip'} . " - " . $data->{'last_ip'} )
              or return;
        }

    }

    return bless( $data, $package );
}

=head2 network()

returns the network IP address (if available)

=over 4

=item Parameter

    none

=item Return

    string:
        IP address of the network
        or undef, if network IP address is not defined 
        (only range, but no netmask)

=back

=cut

sub network
{
    my $self = shift;

    if ( $self->{'ip_network'} ) {
        return $self->{'network'};
    } else {
        return;
    }
}

=head2 returnAddress

compability wrapper

=over 4

=item Parameter

    none

=item Return

    string:
        - network address of the ip range

=back

=cut

sub returnAddress
{
    my $self = shift;

    return $self->network() || $self->first();
}

=head2 netmask()

returns the netmask of the IP network (if available)

=over 4

=item Parameter

    none

=item Return

    string:
        IP address of the netmask
        or undef, if network IP address is not defined 
        (only range, but no netmask)

=back

=cut

sub netmask
{
    my $self = shift;

    if ( $self->{'netmask'} ) {
        return $self->{'netmask'};
    } else {
        return;
    }
}

=head2 broadcast()

returns the broadcast IP address of the network (if available)

=over 4

=item Parameter

    none

=item Return

    string:
        braodcast IP address of the network
        or undef, if network IP address is not defined 
        (only range, but no netmask)

=back

=cut

sub broadcast
{
    my $self = shift;

    if ( $self->{'ip_network'} ) {
        return $self->{'ip_network'}->broadcast();
    } else {
        return;
    }
}

=head2 first()

gets first IP address of the ip range.
If no ip range is defined, it gets the network ip.

=over 4

=item Parameter

    none

=item Return

    string:
        - first IP address of the ip range

=back

=cut

sub first
{
    my $self = shift;

    return $self->{'first_ip'};
}

=head2 last()

gets last IP address of the ip range.
If no ip range is defined, it gets the broadcast ip.

=over 4

=item Parameter

    none

=item Return

    string:
        - last IP address of the ip range

=back

=cut

sub last
{
    my $self = shift;

    return $self->{'last_ip'};
}

=head2 is_in_range( $start_ip, $end_ip )

checks, if instance ip range is inside $start_ip and $end_ip.

=over 4

=item Parameter

    - string: start IP address
    - string: end IP address

=item Return

    boolean

=back

=cut

sub is_in_range
{
    my $self = shift;

    my $from = shift;
    my $to   = shift;

    #     my ( $a1, $a2, $a3, $a4 ) = split( /\./, $from );
    #     my ( $b1, $b2, $b3, $b4 ) = split( /\./, $self->{'IP'} );
    #     my ( $c1, $c2, $c3, $c4 ) = split( /\./, $to );
    #
    #     if ( $a1 <= $b1 && $b1 <= $c1 ) {
    #         if ( $a2 <= $b2 && $b2 <= $c2 ) {
    #             if ( $a3 <= $b3 && $b3 <= $c3 ) {
    #                 if ( $a4 <= $b4 && $b4 <= $c4 ) {
    #                     return 1;
    #                 }
    #             }
    #         }
    #     }
    #     return 0;

    my $check_ip_range = Net::IP->new( $from . " - " . $to ) || return;

    my $result = $self->{'ip_range'}->overlaps($check_ip_range) < 0;
    return $result;
}

=head2 contains( $ip )

checks, if IP address is contained in the IP range

=over 4

=item Parameter

    - string: IP address or IP address range ( IP - IP )

=item Return

    boolean

=back

=cut

sub contains
{
    my $self = shift;

    my $ip = shift;

    my $result = $self->{'ip_range'}->overlaps( Net::IP->new($ip) ) < 0;

    return $result;
}

=head2 overlaps

checks, if a own IP range and parameter overlaps.
Based on Net::IP and uses their return codes.

=over 4

=item Parameter

    - string: IP address or IP address range ( IP - IP )

=item Return

    Net::IP return codes:
        $NO_OVERLAP         (no overlap)
        $IP_PARTIAL_OVERLAP (overlap)
        $IP_A_IN_B_OVERLAP  (range1 is included in range2)
        $IP_B_IN_A_OVERLAP  (range2 is included in range1)
        $IP_IDENTICAL       (range1 == range2) 
        or undef (problem)

=back

=cut

sub overlaps
{
    my $self = shift;

    my $ip = shift;

    return $self->{'ip_range'}->overlaps( Net::IP->new($ip) );
}

=head2 match

Returns a true if the IP number $ip matches the given network. That is, a true value is returned if $ip is between base() amd broadcast(). For example, if we have the network 192.168.1.0/24, then 

  192.168.0.255 => 0
  192.168.1.0   => "0 "
  192.168.1.1   => 1
  ...
  192.168.1.255 => 255

 $ip should be a dotted-quad (eg: ``192.168.66.3'') 

It just happens that the return value is the position within the block. Since zero is a legal position, the true string ``0 '' is returned in it's place. ``0 '' is numerically zero though. When wanting to know the position inside the block, a good idiom is: 

  $pos = $block->match($ip) or die;
  $pos += 0;

=over 4

=item Parameter

    - string: IP address

=item Return

    integer

=back

=cut

sub match
{
    my $self = shift;

    my $ip = shift;

    if ( $self->{'ip_network'} ) {
        return $self->{'ip_network'}->match($ip);
    } else {
        return;
    }
}

=head2 nth

checks, if a LDAP entry uses a specific objectc

=over 4

=item Parameter

    - integer

=item Return

    string:
        - nth IP of the network 
            or undef on error

=back

=cut

sub nth
{
    my $self = shift;

    my $counter = shift;

    if ( $self->{'ip_network'} ) {
        return $self->{'ip_network'}->nth($counter);
    } else {
        return;
    }
}

=head2 getIpList

get list of all IP addresses in the IP range

=over 4

=item Parameter

    - NONE

=item Return

    list of IP addresses (strings)

=back

=cut

sub getIpList
{
    my $self = shift;

    my $ip_version = $self->{'ip_version'};

    # convert start and end address to integers
    my $int_start = Net::IP->new( $self->first() )->intip();
    my $int_end   = Net::IP->new( $self->last() )->intip();

    #print "start: $int_start\nend: $int_end\n";

    my @ip_addresses = ();

    # for loop throw the integers of the ip range
    for ( my $i = $int_start ; $i <= $int_end ; $i++ ) {

        # convert integer back to ip
        my $ip = ip_bintoip( ip_inttobin( $i, $ip_version ), $ip_version );
        push @ip_addresses, $ip;

        #print $ip . "\n";
    }

    #print "( " . join( ", ", @ip_addresses ) . " )\n";

    return @ip_addresses;
}

=head2 getIpIterator

returns an iterator for the ip addresses in C<$self>

=head3 Usage

    my $iterater = $ip->getIpIterator();
    while ( my $ip_address = $iterator->() ) {
        #do something with $ip_address
    }

=over 4

=item Parameter

    - None

=item Return

   - reference to anonymous subroutine (so called iterator)
   
=back

=cut

sub getIpIterator
{
    my $self       = shift;
    my $ip_version = $self->{'ip_version'};

    # convert start and end address to integers
    my $int_current = Net::IP->new( $self->first() )->intip();
    my $int_end     = Net::IP->new( $self->last() )->intip();

    return sub {
        my $ip =
          ip_bintoip( ip_inttobin( $int_current, $ip_version ), $ip_version );
        if ( ( !$ip ) || ( $int_current > $int_end ) ) {

            #end of interval
            return;
        }
        ++$int_current;
        return $ip;
      }
}

=head2 reverse_ip

    return the IP in reverse IP (in.addr.) format.
    Used for reverse zone file names.

=over 4

=item Parameter

    - none

=item Return

    - reverse IP (in.addr. format)

=back

=cut

sub reverse_ip
{
    my $self = shift;

    return $self->{'ip_range'}->reverse_ip();
}

=head2 getReverseZones

Returns the list of /24 reverse zone names. For a network with
a netmask less than /24 will return multiple zone names, for
a netmask greater or equal /24 or single IPs, only one reverse
zone name will be returned.

=over 4

=item Parameter

    - none

=item Return

array of reverse zone names

=back

=cut

sub getReverseZones
{
    my $self = shift;

    my @rev_zones = ();

    if ( !ref $self->{'ip_network'} ) {
        my $logger = Log::Log4perl->get_logger();
        $logger->error( "ip_network is not in this object for "
              . $self->{'first_ip'}
              . ", so no reverse zones will be returned" );
        return;
    }
    my @rev_data = $self->{'ip_network'}->inaddr();
    while (@rev_data) {
        my @rev_zone = splice( @rev_data, 0, 3 );
        push( @rev_zones, $rev_zone[0] );
    }
    return @rev_zones;
}

=head2 error

return the error message of the last error

=over 4

=item Parameter

    none

=item Return

    string: error message

=back

=cut

sub error
{
    my $self = shift;

    return $self->{'error'};
}

=head2 getNetIpInstance

checks, if a LDAP entry uses a specific objectc

=over 4

=item Parameter

    none

=item Return

    inernal used Net::IP instance

=back

=cut

sub getNetIpInstance
{
    my $self = shift;

    return $self->{'ip_range'};
}

1;
